@@ -28,7 +28,7 @@ from ssh2.utils import find_eol
 from ssh2.exceptions import AgentConnectionError, AgentListIdentitiesError, \
     AgentAuthenticationError, AgentGetIdentityError
 
-from ..common import _validate_pkey_path
+from ..common import _validate_pkey
 from ...constants import DEFAULT_RETRIES, RETRY_DELAY
 from ..reader import ConcurrentRWBuffer
 from ...exceptions import UnknownHostError, AuthenticationError, \
@@ -182,12 +182,15 @@ class BaseSSHClient(object):
         self.session = None
         self._host = proxy_host if proxy_host else host
         self._port = proxy_port if proxy_port else self.port
-        self.pkey = _validate_pkey_path(pkey, self.host)
+        self.pkey = _validate_pkey(pkey)
         self.identity_auth = identity_auth
         self._keepalive_greenlet = None
         self.ipv6_only = ipv6_only
         self._init()
 
+    def _pkey_from_memory(self, pkey_data):
+        raise NotImplementedError
+
     def _init(self):
         self._connect(self._host, self._port)
         self._init_session()
@@ -204,8 +207,8 @@ class BaseSSHClient(object):
             if retries < self.num_retries:
                 sleep(self.retry_delay)
                 return self._auth_retry(retries=retries+1)
-            msg = "Authentication error while connecting to %s:%s - %s"
-            raise AuthenticationError(msg, self.host, self.port, ex)
+            msg = "Authentication error while connecting to %s:%s - %s - retries %s/%s"
+            raise AuthenticationError(msg, self.host, self.port, ex, retries, self.num_retries)
 
     def disconnect(self):
         raise NotImplementedError
@@ -281,13 +284,25 @@ class BaseSSHClient(object):
                                           host, str(ex.args[1]), retries,
                                           self.num_retries)
             raise unknown_ex from ex
-        family, _type, proto, _, sock_addr = addr_info[0]
+        for i, (family, _type, proto, _, sock_addr) in enumerate(addr_info):
+            try:
+                return self._connect_socket(family, _type, proto, sock_addr, host, port, retries)
+            except ConnectionRefusedError as ex:
+                if i+1 == len(addr_info):
+                    logger.error("No available addresses from %s", [addr[4] for addr in addr_info])
+                    ex.args += (host, port)
+                    raise
+                continue
+
+    def _connect_socket(self, family, _type, proto, sock_addr, host, port, retries):
         self.sock = socket.socket(family, _type)
         if self.timeout:
             self.sock.settimeout(self.timeout)
         logger.debug("Connecting to %s:%s", host, port)
         try:
             self.sock.connect(sock_addr)
+        except ConnectionRefusedError:
+            raise
         except sock_error as ex:
             logger.error("Error connecting to host '%s:%s' - retry %s/%s",
                          host, port, retries, self.num_retries)
@@ -309,7 +324,7 @@ class BaseSSHClient(object):
                 "Trying to authenticate with identity file %s",
                 identity_file)
             try:
-                self._pkey_auth(identity_file, password=self.password)
+                self._pkey_file_auth(identity_file, password=self.password)
             except Exception as ex:
                 logger.debug(
                     "Authentication with identity file %s failed with %s, "
@@ -331,8 +346,8 @@ class BaseSSHClient(object):
     def auth(self):
         if self.pkey is not None:
             logger.debug(
-                "Proceeding with private key file authentication")
-            return self._pkey_auth(self.pkey, password=self.password)
+                "Proceeding with private key authentication")
+            return self._pkey_auth(self.pkey)
         if self.allow_agent:
             try:
                 self._agent_auth()
@@ -364,7 +379,17 @@ class BaseSSHClient(object):
     def _password_auth(self):
         raise NotImplementedError
 
-    def _pkey_auth(self, pkey_file, password=None):
+    def _pkey_auth(self, pkey):
+        _pkey = pkey
+        if isinstance(pkey, str):
+            logger.debug("Private key is provided as str, loading from private key file path")
+            with open(pkey, 'rb') as fh:
+                _pkey = fh.read()
+        elif isinstance(pkey, bytes):
+            logger.debug("Private key is provided in bytes, using as private key data")
+        return self._pkey_from_memory(_pkey)
+
+    def _pkey_file_auth(self, pkey_file, password=None):
         raise NotImplementedError
 
     def _open_session(self):
