@@ -20,7 +20,7 @@ import os
 from ..exceptions import PKeyFileError
 
 
-def _validate_pkey_path(pkey, host=None):
+def _validate_pkey_path(pkey):
     if pkey is None:
         return
     pkey = os.path.normpath(os.path.expanduser(pkey))
@@ -31,3 +31,11 @@ def _validate_pkey_path(pkey, host=None):
         ex = PKeyFileError(msg, pkey)
         raise ex
     return pkey
+
+
+def _validate_pkey(pkey):
+    if pkey is None:
+        return
+    if isinstance(pkey, str):
+        return _validate_pkey_path(pkey)
+    return pkey
