@@ -75,7 +75,8 @@ class SSHClient(BaseSSHClient):
         :param pkey: Private key file path to use for authentication. Path must
           be either absolute path or relative to user home directory
           like ``~/<path>``.
-        :type pkey: str
+          Bytes type input is used as private key data for authentication.
+        :type pkey: str or bytes
         :param num_retries: (Optional) Number of connection and authentication
           attempts before the client gives up. Defaults to 3.
         :type num_retries: int
@@ -239,12 +240,19 @@ class SSHClient(BaseSSHClient):
     def _agent_auth(self):
         self.session.agent_auth(self.user)
 
-    def _pkey_auth(self, pkey_file, password=None):
+    def _pkey_file_auth(self, pkey_file, password=None):
         self.session.userauth_publickey_fromfile(
             self.user,
             pkey_file,
             passphrase=password if password is not None else b'')
 
+    def _pkey_from_memory(self, pkey_data):
+        self.session.userauth_publickey_frommemory(
+            self.user,
+            pkey_data,
+            passphrase=self.password if self.password is not None else b'',
+        )
+
     def _password_auth(self):
         self.session.userauth_password(self.user, self.password)
 
@@ -590,6 +598,7 @@ class SSHClient(BaseSSHClient):
                 total += size
                 local_fh.write(data)
         finally:
+            local_fh.flush()
             local_fh.close()
             file_chan.close()
 
@@ -651,13 +660,16 @@ class SSHClient(BaseSSHClient):
             raise SCPError(msg, remote_file, self.host, ex)
         try:
             with open(local_file, 'rb', 2097152) as local_fh:
-                for data in local_fh:
+                data = local_fh.read(self._BUF_SIZE)
+                while data:
                     self.eagain_write(chan.write, data)
+                    data = local_fh.read(self._BUF_SIZE)
         except Exception as ex:
             msg = "Error writing to remote file %s on host %s - %s"
             logger.error(msg, remote_file, self.host, ex)
             raise SCPError(msg, remote_file, self.host, ex)
         finally:
+            self._eagain(chan.flush)
             chan.close()
 
     def _sftp_openfh(self, open_func, remote_file, *args):
